import os

from collections import OrderedDict

from sendfile import sendfile

from django.conf import settings
from django.utils.timezone import now

from rest_framework import viewsets, serializers
from rest_framework.response import Response
from rest_framework.exceptions import NotFound

from daiquiri.core.viewsets import RowViewSetMixin
from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.utils import get_client_ip
from daiquiri.stats.models import Record

from .models import Collection, ArchiveJob

from .permissions import HasPermission


class RowViewSet(RowViewSetMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    def list(self, request, *args, **kwargs):
        # fetch rows from the database
        column_names = [column['name'] for column in settings.ARCHIVE_COLUMNS]

        # get the row query params from the request
        ordering, page, page_size, search, filters = self._get_query_params(column_names)

        # get database adapter
        adapter = DatabaseAdapter()

        # get the schema_name and the table_name from the settings
        schema_name = settings.ARCHIVE_SCHEMA
        table_name = settings.ARCHIVE_TABLE

        # get collecions for this user and add them to the filters
        collections = [collection.name for collection in Collection.objects.filter_by_access_level(request.user)]
        filters['collection'] = collections

        # query the database for the total number of rows
        count = adapter.count_rows(schema_name, table_name, column_names, search, filters)

        # query the paginated rowset
        results = adapter.fetch_rows(schema_name, table_name, column_names, ordering, page, page_size, search, filters)

        # return ordered dict to be send as json
        return Response(OrderedDict((
            ('count', count),
            ('results', results),
            ('next', self._get_next_url(page, page_size, count)),
            ('previous', self._get_previous_url(page))
        )))


class ColumnViewSet(viewsets.ViewSet):
    permission_classes = (HasPermission, )

    def list(self, request, *args, **kwargs):

        return Response(settings.ARCHIVE_COLUMNS)


class FileViewSet(viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    serializer_class = serializers.Serializer

    def retrieve(self, request, pk=None):
        # get database adapter
        adapter = DatabaseAdapter()

        # get the schema_name and the table_name from the settings
        schema_name = settings.ARCHIVE_SCHEMA
        table_name = settings.ARCHIVE_TABLE

        # get collecions for this user
        collections = [collection.name for collection in Collection.objects.filter_by_access_level(request.user)]

        # fetch the path for this file from the database
        row = adapter.fetch_row(schema_name, table_name, ['path'], filters={
            'id': pk,
            'collection': collections
        })

        if row:
            if request.GET.get('download', True):
                # create a stats record for this download
                Record.objects.create(
                    time=now(),
                    resource_type='ARCHIVE_DOWNLOAD',
                    resource=row[0],
                    client_ip=get_client_ip(request),
                    user=request.user if request.user.is_authenticated else None
                )

                # send the file to the client
                file_path = os.path.join(settings.ARCHIVE_BASE_PATH, row[0])
                return sendfile(request, file_path, attachment=True)
            else:
                # send an empty response
                return Response()

        # if the file was not found, return 404
        raise NotFound()


class ArchiveViewSet(viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    serializer_class = serializers.Serializer

    def retrieve(self, request, pk=None):
        try:
            archive_job = ArchiveJob.objects.filter_by_owner(request.user).get(pk=pk)
        except ArchiveJob.DoesNotExist:
            raise NotFound

        if archive_job.phase == archive_job.PHASE_COMPLETED and request.GET.get('download', True):
            return sendfile(request, archive_job.file_path, attachment=True)
        else:
            return Response(archive_job.phase)

    def create(self, request):
        try:
            archive_job = ArchiveJob.objects.filter_by_owner(request.user).get(data=request.data)

            # check if the file was lost
            if archive_job.phase == archive_job.PHASE_COMPLETED and \
                    not os.path.isfile(archive_job.file_path):

                # set the phase back to pending so that the file is recreated
                archive_job.phase = archive_job.PHASE_PENDING
                archive_job.process()
                archive_job.save()
                archive_job.run()

        except ArchiveJob.DoesNotExist:
            archive_job = ArchiveJob(
                owner=(None if self.request.user.is_anonymous() else self.request.user),
                client_ip=get_client_ip(self.request),
                data=request.data
            )
            archive_job.process()
            archive_job.save()
            archive_job.run()

        return Response({
            'id': archive_job.id
        })



from django.conf import settings
from django.http import FileResponse

from rest_framework.exceptions import NotFound, ValidationError

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.adapter.stream import BaseServiceAdapter
from daiquiri.core.utils import import_class
from daiquiri.core.generators import generate_votable


def ConeSearchAdapter():
    return import_class(settings.CONESEARCH_ADAPTER)()


class BaseConeSearchAdapter(BaseServiceAdapter):

    keys = ['RA', 'DEC', 'SR']

    ranges = {
        'RA': {
            'min': 0,
            'max': 360
        },
        'DEC': {
            'min': -90,
            'max': 90
        },
        'SR': {
            'min': 0,
            'max': 10
        }
    }

    defaults = {}

    columns = [
        {
            'name': 'id',
            'ucd': 'meta.id; meta.main',
            'datatype': 'char',
            'arraysize': '*'
        },
        {
            'name': 'RA',
            'ucd': 'pos.eq.ra; meta.main',
            'datatype': 'double'
        },
        {
            'name': 'DEC',
            'ucd': 'pos.eq.dec; meta.main',
            'datatype': 'double'
        }
    ]

    def stream(self):
        return FileResponse(generate_votable(DatabaseAdapter().fetchall(self.sql, self.args), self.columns), content_type='application/xml')


class SimpleConeSearchAdapter(BaseConeSearchAdapter):

    def clean(self, request, resource):
        if resource != settings.CONESEARCH_TABLE:
            raise NotFound()

        adapter = DatabaseAdapter()

        self.sql = '''
SELECT %(id_field)s, %(ra_field)s, %(dec_field)s
FROM %(schema)s.%(table)s
WHERE
    %(ra_field)s BETWEEN (%%(RA)s - 0.5 * %%(SR)s) AND (%%(RA)s + 0.5 * %%(SR)s)
AND
    %(dec_field)s BETWEEN (%%(DEC)s - 0.5 * %%(SR)s) AND (%%(DEC)s + 0.5 * %%(SR)s)
''' % {
            'id_field': adapter.escape_identifier('id'),
            'ra_field': adapter.escape_identifier('ra'),
            'dec_field': adapter.escape_identifier('dec'),
            'schema': settings.CONESEARCH_SCHEMA,
            'table': settings.CONESEARCH_TABLE
        }

        self.args, errors = self.parse_query_dict(request)

        if errors:
            raise ValidationError(errors)



import logging
import os

from sendfile import sendfile

from django.conf import settings
from django.utils.timezone import now

from daiquiri.core.utils import get_client_ip
from daiquiri.stats.models import Record

from .models import Directory

logger = logging.getLogger(__name__)


def check_file(user, file_path):
    # loop over all directories beginning with the hights depth and return as soon as a directory matches
    for directory in Directory.objects.order_by('-depth'):
        if os.path.normpath(file_path).startswith(directory.path):
            return Directory.objects.filter_by_access_level(user).filter(pk=directory.pk).exists()


def search_file(search_path):
    base_path = os.path.normpath(settings.FILES_BASE_PATH)

    # look for the file in all directory below the base path
    results = set()
    for directory_path, _, _ in os.walk(base_path):
        normalized_file_path = normalize_file_path(directory_path, search_path)
        absolute_file_path = os.path.join(directory_path, normalized_file_path)

        if os.path.isfile(absolute_file_path):
            results.add(absolute_file_path)

    if len(results) == 1:
        # subtract the base path and return
        file_path = results.pop().split(base_path, 1)[1].lstrip('/')

        logger.debug('%s => %s', search_path, file_path)
        return file_path
    elif len(results) > 1:
        logger.debug('search_path = %s found more than once', search_path)
        return None
    else:
        logger.debug('search_path = %s not found', search_path)
        return None


def send_file(request, file_path, search=None):
    # create a stats record for this download
    resource = {
        'file_path': file_path
    }
    if search:
        resource['search'] = search

    Record.objects.create(
        time=now(),
        resource_type='FILE',
        resource=resource,
        client_ip=get_client_ip(request),
        user=request.user if request.user.is_authenticated else None
    )

    # send the file to the client
    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)
    return sendfile(request, absolute_file_path, attachment=True)


def normalize_file_path(directory_path, file_path):

    directory_path_tokens = os.path.normpath(directory_path).split(os.path.sep)
    file_path_tokens = os.path.normpath(file_path).split(os.path.sep)

    match = 0
    for i in range(len(file_path_tokens)):
        if file_path_tokens[:i] == directory_path_tokens[-i:]:
            match = i

    return os.path.join(*file_path_tokens[match:])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-23 11:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Contribution',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, verbose_name='Title')),
                ('abstract', models.TextField(blank=True, null=True, verbose_name='Abstract')),
                ('contribution_type', models.CharField(help_text='Choices are given by settings.MEETINGS_CONTRIBUTION_TYPES', max_length=8, verbose_name='Contribution type')),
                ('accepted', models.BooleanField(default=False, help_text='Designates whether the contribution is accepted.', verbose_name='Accepted')),
            ],
            options={
                'ordering': ('participant', 'title'),
                'verbose_name': 'Contribution',
                'verbose_name_plural': 'Contributions',
                'permissions': (('view_contribution', 'Can view Contribution'),),
            },
        ),
        migrations.CreateModel(
            name='Meeting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Title of the meeting', max_length=256, verbose_name='Title')),
                ('slug', models.SlugField(help_text='Slug for the URL of the meeting', max_length=256, verbose_name='Slug')),
                ('registration_message', models.TextField(blank=True, help_text='Message on registration page', null=True, verbose_name='Registration message')),
                ('participants_message', models.TextField(blank=True, help_text='Message on participants page', null=True, verbose_name='Participants message')),
                ('contributions_message', models.TextField(blank=True, help_text='Message on contributions page', null=True, verbose_name='Contributions message')),
                ('registration_open', models.BooleanField(default=False, help_text='Designates whether the registration page is publicly accessible.', verbose_name='Registration open')),
                ('participants_open', models.BooleanField(default=False, help_text='Designates whether the participants page is publicly accessible.', verbose_name='Participants list open')),
                ('contributions_open', models.BooleanField(default=False, help_text='Designates whether the contributions page is publicly accessible.', verbose_name='Contributions list open')),
            ],
            options={
                'ordering': ('title',),
                'verbose_name': 'Meeting',
                'verbose_name_plural': 'Meetings',
                'permissions': (('view_meeting', 'Can view Meeting'),),
            },
        ),
        migrations.CreateModel(
            name='Participant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=256, verbose_name='First name')),
                ('last_name', models.CharField(max_length=256, verbose_name='Last name')),
                ('email', models.EmailField(max_length=256, verbose_name='Email')),
                ('details', jsonfield.fields.JSONField(blank=True, help_text='Choices are given by settings.MEETINGS_PARTICIPANT_DETAIL_KEYS', null=True, verbose_name='Details')),
                ('registered', models.DateTimeField(help_text='Datetime this participant has submitted his/her registration', verbose_name='Registered on')),
                ('accepted', models.BooleanField(default=False, help_text='Designates whether the participant is accepted.', verbose_name='Accepted')),
                ('meeting', models.ForeignKey(help_text='Meeting this participant has registered for', on_delete=django.db.models.deletion.CASCADE, related_name='participants', to='daiquiri_meetings.Meeting', verbose_name='Meeting')),
            ],
            options={
                'ordering': ('meeting', 'last_name', 'first_name'),
                'verbose_name': 'Participant',
                'verbose_name_plural': 'Participants',
                'permissions': (('view_participant', 'Can view Participant'),),
            },
        ),
        migrations.AddField(
            model_name='contribution',
            name='participant',
            field=models.ForeignKey(help_text='Participant who submitted this contribution', on_delete=django.db.models.deletion.CASCADE, related_name='contributions', to='daiquiri_meetings.Participant', verbose_name='Participant'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-05 13:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_archive', '0002_collection'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='archivejob',
            name='file_path',
        ),
    ]



from django.contrib.auth.models import User
from django.core.management.base import BaseCommand

from daiquiri.query.models import QueryJob


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('username', help='Remove all query jobs of this user.')
        parser.add_argument('-y', action='store_true', help='Answer yes for all questions.')

    def handle(self, *args, **options):

        if options['username'] == 'anonymous':
            jobs = QueryJob.objects.filter_by_owner(None).exclude(phase=QueryJob.PHASE_ARCHIVED)
        else:
            user = User.objects.get(username=options['username'])
            jobs = QueryJob.objects.filter_by_owner(user).exclude(phase=QueryJob.PHASE_ARCHIVED)

        count = jobs.count()

        if count > 0:
            if options['y']:
                archive = True
            else:
                print('%i jobs found. Really archive these jobs? [yes/no]:' % jobs.count())
                archive = raw_input().lower() in ['yes', 'y']

            if archive:
                for job in jobs:
                    job.archive()
        else:
            print('No jobs found for this user.')



from django.test import TestCase

from test_generator.views import TestViewMixin, TestListViewMixin


class ContactViewTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json',
        'contact.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('anonymous', None),
    )

    status_map = {
        'list_view': {
            'admin': 200, 'user': 403, 'anonymous': 302
        }
    }


class MessagesTests(TestListViewMixin, ContactViewTestCase):

    url_names = {
        'list_view': 'contact:messages'
    }


class ContactTests(TestViewMixin, ContactViewTestCase):

    url_names = {
        'create_view': 'contact:contact'
    }

    status_map = {
        'create_view_get': {
            'admin': 200, 'user': 200, 'anonymous': 200
        },
        'create_view_post': {
            'admin': 200, 'user': 200, 'anonymous': 200
        }
    }

    def _test_contact_get(self, username):
        self.assert_view('create_view_get', 'get', 'create_view', username)

    def _test_contact_post(self, username):
        self.assert_view('create_view_post', 'post', 'create_view', username, data={
            'author': 'Tom Test',
            'email': 'test@example.com',
            'subject': 'This is a test',
            'message': 'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est. Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est. Lorem ipsum dolor sit amet.'
        })

    def _test_contact_post_invalid(self, username):
        self.assert_view('create_view_post', 'post', 'create_view', username, data={})

    def _test_contact_cancel(self, username):
        self.assert_view('create_view_post', 'post', 'create_view', username, data={
            'cancel': True
        })



import logging
import csv
import six
import subprocess
import re

from django.conf import settings

from daiquiri.core.generators import generate_csv, generate_votable
from daiquiri.core.utils import get_doi_url

logger = logging.getLogger(__name__)


class BaseDownloadAdapter(object):

    def __init__(self, database_key, database_config):
        self.database_key = database_key
        self.database_config = database_config

    def generate(self, format_key, schema_name, table_name, columns, sources=None, status=None, empty=False):
        # create the final list of arguments subprocess.Popen
        self.set_args(schema_name, table_name)

        prepend = {}
        for ucd, value in settings.DOWNLOAD_PREPEND.items():
            for i, column in enumerate(columns):
                if ucd in column['ucd']:
                    prepend[i] = value

        if sources:
            links = []
            for source in sources:
                links.append({
                    'content-role': 'source',
                    'href': get_doi_url(source['doi']),
                    'name': '%(schema_name)s.%(table_name)s' % source,
                })
        else:
            links = None

        if format_key == 'csv':
            return generate_csv(self.execute(prepend=prepend))

        elif format_key == 'votable':
            return generate_votable(self.execute(prepend=prepend), columns,
                                    resource_name=schema_name, table_name=table_name,
                                    links=links, query_status=status, empty=empty)

        else:
            raise Exception('Not supported.')

    def execute(self, prepend=None):
        # log the arguments
        logger.debug('execute "%s"' % ' '.join(self.args))

        # excecute the subprocess
        try:
            process = subprocess.Popen(self.args, stdout=subprocess.PIPE)

            for line in process.stdout:
                insert_pattern = re.compile('^INSERT INTO .*? VALUES \((.*?)\);')
                insert_result = insert_pattern.match(line.decode())
                if insert_result:
                    line = insert_result.group(1)
                    reader = csv.reader([line], quotechar="'", skipinitialspace=True)
                    row = six.next(reader)

                    if prepend:
                        yield [(prepend[i] + cell if i in prepend else cell) for i, cell in enumerate(row)]
                    else:
                        yield row

        except subprocess.CalledProcessError as e:
            logger.error('Command PIPE returned non-zero exit status: %s' % e)



from django.conf.urls import url, include

from rest_framework import routers

from .views import table
from .viewsets import RowViewSet, ColumnViewSet, ReferenceViewSet

router = routers.DefaultRouter()
router.register(r'rows', RowViewSet, base_name='row')
router.register(r'columns', ColumnViewSet, base_name='column')
router.register(r'references', ReferenceViewSet, base_name='reference')


urlpatterns = [
    url(r'^table/(?P<schema_name>[A-Za-z0-9_]+)/(?P<table_name>[A-Za-z0-9_]+)/$', table, name='table'),

    # rest api
    url(r'^api/', include(router.urls)),
]



from django.contrib.auth.models import User
from django.test import TestCase

from daiquiri.query.utils import check_permissions


class QueryPermissionsTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('anonymous', None),
    )

    def test_columns(self):
        user = User.objects.get(username='admin')
        keywords = []
        tables = [('daiquiri_data_obs', 'stars')]
        columns = [
            ('daiquiri_data_obs', 'stars', 'ra'),
            ('daiquiri_data_obs', 'stars', 'dec')
        ]
        functions = []

        result = check_permissions(user, keywords, tables, columns, functions)

        self.assertEqual(result, [])

    def test_columns_not_found(self):
        user = User.objects.get(username='admin')
        keywords = []
        tables = [('daiquiri_data_obs', 'stars')]
        columns = [
            ('daiquiri_data_obs', 'stars', 'ra'),
            ('daiquiri_data_obs', 'stars', 'not_found')
        ]
        functions = []

        result = check_permissions(user, keywords, tables, columns, functions)

        self.assertEqual(result, ['Column not_found not found.'])

    def test_alias(self):
        user = User.objects.get(username='admin')
        keywords = []
        tables = [('daiquiri_data_obs', 'stars')]
        columns = [
            ('daiquiri_data_obs', 'stars', 'ra'),
            ('daiquiri_data_obs', 'stars', 'dec'),
            (None, None, 'alias')
        ]
        functions = []

        result = check_permissions(user, keywords, tables, columns, functions)

        self.assertEqual(result, [])

    def test_missing_schema(self):
        user = User.objects.get(username='admin')
        keywords = []
        tables = [(None, 'stars')]
        columns = [
            ('daiquiri_data_obs', 'stars', 'ra'),
            ('daiquiri_data_obs', 'stars', 'dec'),
            (None, None, 'alias')
        ]
        functions = []

        result = check_permissions(user, keywords, tables, columns, functions)

        self.assertEqual(result, ['No schema given for table stars.'])

    def test_missing_table(self):
        user = User.objects.get(username='admin')
        keywords = []
        tables = [('daiquiri_data_obs', None)]
        columns = [
            ('daiquiri_data_obs', 'stars', 'ra'),
            ('daiquiri_data_obs', 'stars', 'dec'),
            (None, None, 'alias')
        ]
        functions = []

        result = check_permissions(user, keywords, tables, columns, functions)

        self.assertEqual(result, ['No table given for schema daiquiri_data_obs.'])

    def test_user_schema(self):
        user = User.objects.get(username='user')
        keywords = []
        tables = [('daiquiri_user_user', 'test')]
        columns = [
            (None, None, '*')
        ]
        functions = []

        result = check_permissions(user, keywords, tables, columns, functions)

        self.assertEqual(result, [])



import os

from .base import BASE_DIR

ASYNC = False

IPV4_PRIVACY_MASK = 16
IPV6_PRIVACY_MASK = 32

DOWNLOAD_PREPEND = {}

ARCHIVE_ANONYMOUS = False
ARCHIVE_SCHEMA = 'daiquiri_archive'
ARCHIVE_TABLE = 'files'
ARCHIVE_COLUMNS = [
    {
        'name': 'id',
        'hidden': True
    },
    {
        'name': 'timestamp',
        'label': 'Timestamp'
    },
    {
        'name': 'file',
        'label': 'Filename',
        'ucd': 'meta.file'
    },
    {
        'name': 'collection',
        'hidden': True
    },
    {
        'name': 'path',
        'hidden': True
    }
]
ARCHIVE_BASE_PATH = os.path.join(BASE_DIR, 'files')
ARCHIVE_DOWNLOAD_DIR = os.path.join(BASE_DIR, 'download')

AUTH_SIGNUP = False
AUTH_WORKFLOW = None
AUTH_DETAIL_KEYS = []

CONESEARCH_ADAPTER = 'daiquiri.conesearch.adapter.SimpleConeSearchAdapter'
CONESEARCH_ANONYMOUS = False

CUTOUT_ADAPTER = 'daiquiri.cutout.adapter.SimpleCutOutAdapter'
CUTOUT_ANONYMOUS = False

FILES_BASE_PATH = os.path.join(BASE_DIR, 'files')
FILES_SEARCH_URL = None

MEETINGS_CONTRIBUTION_TYPES = [
    (None, 'no contribution'),
    ('talk', 'Talk'),
    ('poster', 'Poster')
]
MEETINGS_PARTICIPANT_DETAIL_KEYS = []
MEETINGS_ABSTRACT_MAX_LENGTH = 2000

METADATA_COLUMN_PERMISSIONS = False

QUERY_ANONYMOUS = False
QUERY_USER_SCHEMA_PREFIX = 'daiquiri_user_'
QUERY_QUOTA = {
    'anonymous': '100Mb',
    'user': '100Mb',
    'users': {},
    'groups': {}
}
QUERY_MAX_ACTIVE_JOBS = {
    'anonymous': '1'
}
QUERY_QUEUES = [
    {
        'key': 'default',
        'label': 'Default',
        'timeout': 10,
        'priority': 1
    }
]
QUERY_LANGUAGES = [
    {
        'key': 'adql',
        'version': 2.0,
        'label': 'ADQL',
        'description': '',
        'quote_char': '"'
    }
]
QUERY_FORMS = [
    {
        'key': 'sql',
        'label': 'SQL query',
        'service': 'query/js/forms/sql.js',
        'template': 'query/query_form_sql.html'
    }
]
QUERY_DROPDOWNS = [
    {
        'key': 'simbad',
        'service': 'query/js/dropdowns/simbad.js',
        'template': 'query/query_dropdown_simbad.html',
        'options': {
            'url': 'http://simbad.u-strasbg.fr/simbad/sim-id'
        }
    },
    {
        'key': 'vizier',
        'service': 'query/js/dropdowns/vizier.js',
        'template': 'query/query_dropdown_vizier.html',
        'options': {
            'url': 'http://vizier.u-strasbg.fr/viz-bin/votable',
            'catalogs': ['I/322A', 'I/259']
        }
    }
]
QUERY_DOWNLOAD_DIR = os.path.join(BASE_DIR, 'download')
QUERY_DEFAULT_DOWNLOAD_FORMAT = 'votable'
QUERY_DOWNLOAD_FORMATS = [
    {
        'key': 'csv',
        'extension': 'csv',
        'content_type': 'text/csv',
        'label': 'Comma separated Values',
        'help': 'A text file with a line for each row of the table. The fields are delimited by a comma and quoted by double quotes. Use this option for a later import into a spreadsheed application or a custom script. Use this option if you are unsure what to use.'
    },
    {
        'key': 'votable',
        'extension': 'xml',
        'content_type': 'application/xml',
        'label': 'IVOA VOTable XML file - TABLEDATA serialization',
        'help': 'A XML file using the IVOA VOTable format. Use this option if you intend to use VO compatible software to further process the data.'
    }
]

SERVE_DOWNLOAD_DIR = os.path.join(BASE_DIR, 'download')
SERVE_RESOLVER = None

STATS_RESOURCE_TYPES = [
    {
        'key': 'ARCHIVE_DOWNLOAD',
        'label': 'Archive downloads'
    },
    {
        'key': 'CONESEARCH',
        'label': 'Performed cone searches'
    },
    {
        'key': 'CUTOUT',
        'label': 'Performed cutouts'
    },
    {
        'key': 'FILE',
        'label': 'File downloads'
    },
    {
        'key': 'QUERY_JOB',
        'label': 'Query jobs'
    }
]

UWS_RESOURCES = []

TAP_SCHEMA = 'TAP_SCHEMA'

WORDPRESS_URL = '/cms/'
WORDPRESS_CLI = '/opt/wp-cli/wp'
WORDPRESS_PATH = '/opt/wordpress'



import csv
import io
import sys


def generate_csv(generator):
    if sys.version_info.major >= 3:
        io_class = io.StringIO
    else:
        io_class = io.BytesIO

    for row in generator:
        if row:
            f = io_class()
            csv.writer(f, quotechar='"').writerow(row)
            yield f.getvalue()


def generate_votable(generator, fields, resource_name=None, table_name=None, links=None, query_status=None, empty=None):

        yield '''<?xml version="1.0"?>
<VOTABLE version="1.3"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns="http://www.ivoa.net/xml/VOTable/v1.3"
    xmlns:stc="http://www.ivoa.net/xml/STC/v1.30">'''

        if resource_name:
            yield '''
    <RESOURCE name="%(name)s" type="results">''' % {'name': resource_name}
        else:
            yield '''
    <RESOURCE type="results">'''

        if query_status == 'OK':
            yield '''
        <INFO name="QUERY_STATUS" value="OK" />'''
        elif query_status == 'OVERFLOW':
            yield '''
        <INFO name="QUERY_STATUS" value="OVERFLOW" />'''

        if links:
            for link in links:
                attrs = []
                for key in ['name', 'href', 'content-type', 'content-role']:
                    if key in link and link[key]:
                        attrs.append('%s="%s"' % (key, link[key]))

                yield '''
        <LINK %s />''' % ' '.join(attrs)

        if table_name:
            yield '''
        <TABLE name="%(table)s">''' % {'table': table_name}
        else:
            yield '''
        <TABLE>'''

        for field in fields:
            attrs = []
            for key in ['name', 'arraysize', 'unit', 'ucd', 'utype']:
                if key in field and field[key]:
                    attrs.append('%s="%s"' % (key, field[key]))

            if field['datatype'] in ['char', 'unsignedByte', 'short', 'int', 'long', 'float', 'double']:
                attrs.append('datatype="%s"' % field['datatype'])
            else:
                attrs.append('xtype="%s"' % field['datatype'])

            yield '''
            <FIELD %s />''' % ' '.join(attrs)

        if not empty:
            yield '''
            <DATA>
                <TABLEDATA>'''

            # write rows of the table yielded by the generator
            for row in generator:
                yield '''
                    <TR>
                        <TD>%s</TD>
                    </TR>''' % '''</TD>
                        <TD>'''.join([('' if cell == 'NULL' else str(cell)) for cell in row])

            yield '''
                </TABLEDATA>
            </DATA>'''
        yield '''
        </TABLE>
    </RESOURCE>
</VOTABLE>
'''



from django import forms
from django.utils.translation import ugettext as _

from .models import ContactMessage


class ContactForm(forms.ModelForm):

    use_required_attribute = False

    class Meta:
        model = ContactMessage
        fields = ['author', 'email', 'subject', 'message']
        widgets = {
            'author': forms.TextInput(attrs={'placeholder': _('Your name')}),
            'email': forms.TextInput(attrs={'placeholder': _('Your email address')}),
            'subject': forms.TextInput(attrs={'placeholder': _('Subject')}),
            'message': forms.Textarea(attrs={'placeholder': _('Your message')}),
        }
        labels = {
            'author': _('Your name'),
            'email': _('Please enter your email address so our answer will reach you'),
            'message': _('Your message')
        }



import logging

from django.test import TestCase

from daiquiri.core.adapter import DatabaseAdapter

logger = logging.getLogger(__name__)


class CoreAdapterTestCase(TestCase):

    def test_fetch_table(self):
        rows = DatabaseAdapter().fetch_table('daiquiri_data_obs', 'stars')
        self.assertEqual(rows['type'], 'table')

    def test_fetch_row(self):
        row = DatabaseAdapter().fetch_row('daiquiri_data_obs', 'stars', column_names=None, search=None, filters={
            'id': '4551299946478123136'
        })
        self.assertEqual(row[0], 4551299946478123136)

    def test_fetch_rows(self):
        rows = DatabaseAdapter().fetch_rows('daiquiri_data_obs', 'stars')
        self.assertEqual(len(rows), 10)
        self.assertEqual(len(rows[0]), 4)
        self.assertEqual(rows[0][0], 1714709274237701248)

    def test_fetch_rows_filter(self):
        rows = DatabaseAdapter().fetch_rows('daiquiri_data_sim', 'halos', filters={
            'id': '85000000000'
        })
        self.assertEqual(len(rows), 1)
        self.assertEqual(len(rows[0]), 8)
        self.assertEqual(rows[0][0], 85000000000)

    def test_fetch_nrows(self):
        nrows = DatabaseAdapter().fetch_nrows('daiquiri_data_obs', 'stars')
        self.assertEqual(nrows, 10000)

    def test_fetch_size(self):
        size = DatabaseAdapter().fetch_size('daiquiri_data_obs', 'stars')
        self.assertGreater(size, 250000)

    def test_fetch_columns(self):
        columns = DatabaseAdapter().fetch_columns('daiquiri_archive', 'files')
        self.assertEqual(len(columns), 7)
        self.assertTrue(columns[0]['indexed'])

    def test_fetch_column(self):
        column = DatabaseAdapter().fetch_column('daiquiri_archive', 'files', 'id')
        self.assertTrue(column['datatype'].startswith('char'))

    def test_fetch_column_names(self):
        columns = DatabaseAdapter().fetch_column_names('daiquiri_data_obs', 'stars')
        self.assertEqual(columns[0], 'id')

    def test_count_rows(self):
        row = DatabaseAdapter().count_rows('daiquiri_data_obs', 'stars')
        self.assertEqual(row, 10000)



import logging

from django.shortcuts import render
from django.http import Http404, HttpResponseRedirect
from django.urls import reverse

from .models import Meeting, Participant, Contribution
from .forms import ParticipantForm, ContributionForm
from .utils import send_registration_mails

logger = logging.getLogger(__name__)


def registration(request, slug):
    try:
        meeting = Meeting.objects.get(slug=slug)
    except Meeting.DoesNotExist:
        raise Http404

    if meeting.registration_open:
        participant_form = ParticipantForm(request.POST or None, meeting=meeting)
        contribution_form = ContributionForm(request.POST or None)

        if request.method == 'POST':
            if participant_form.is_valid() and contribution_form.is_valid():
                participant = participant_form.save()

                logger.info('participant \'%s\' registered.' % (participant.full_name, ))

                if contribution_form.cleaned_data['contribution_type']:
                    contribution = contribution_form.save(commit=False)
                    contribution.participant = participant
                    contribution.save()

                    send_registration_mails(request, meeting, participant, contribution)
                else:
                    send_registration_mails(request, meeting, participant)

                return HttpResponseRedirect(reverse('meetings:registration_done', kwargs={
                    'slug': slug
                }))

        return render(request, 'meetings/registration.html', {
            'meeting': meeting,
            'participant_form': participant_form,
            'contribution_form': contribution_form
        })
    else:
        return render(request, 'meetings/registration_closed.html', {})


def registration_done(request, slug):
    try:
        meeting = Meeting.objects.get(slug=slug)
    except Meeting.DoesNotExist:
        raise Http404

    if meeting.registration_open:
        return render(request, 'meetings/registration_done.html', {
            'meeting': meeting,
        })
    else:
        return render(request, 'meetings/registration_closed.html', {})


def participants(request, slug):
    try:
        meeting = Meeting.objects.get(slug=slug)
    except Meeting.DoesNotExist:
        raise Http404

    if meeting.participants_open:
        return render(request, 'meetings/participants.html', {
            'meeting': meeting,
            'participants': Participant.objects.filter(meeting=meeting, accepted=True)
        })
    else:
        return render(request, 'meetings/participants_closed.html', {})


def contributions(request, slug):
    try:
        meeting = Meeting.objects.get(slug=slug)
    except Meeting.DoesNotExist:
        raise Http404

    if meeting.contributions_open:
        return render(request, 'meetings/contributions.html', {
            'meeting': meeting,
            'contributions': Contribution.objects.filter(participant__meeting=meeting, accepted=True)
        })
    else:
        return render(request, 'meetings/contributions_closed.html', {})



from django.shortcuts import render
from django.http import Http404

from .utils import get_columns


def table(request, schema_name, table_name):

    columns = get_columns(request.user, schema_name, table_name)

    if columns:
        return render(request, 'serve/table.html', {
            'schema': schema_name,
            'table': table_name
        })

    # if nothing worked, return 404
    raise Http404



from django.dispatch import receiver

from daiquiri.auth.signals import (
    user_created,
    user_updated,
    user_groups_updated
)

from .utils import (
    update_wordpress_user,
    update_wordpress_role
)


@receiver(user_created)
def user_created_handler(sender, **kwargs):
    update_wordpress_user(kwargs['user'])


@receiver(user_updated)
def user_updated_handler(sender, **kwargs):
    update_wordpress_user(kwargs['user'])


@receiver(user_groups_updated)
def user_groups_updated_handler(sender, **kwargs):
    update_wordpress_role(kwargs['user'])



from django.contrib import admin

from .models import Profile


class ProfileAdmin(admin.ModelAdmin):
    search_fields = ('__str__', )
    list_display = ('__str__', 'is_pending', 'is_confirmed')
    readonly_fields = ('user', )

admin.site.register(Profile, ProfileAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-23 16:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_meetings', '0002_meeting_registration_done_message'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contribution',
            name='abstract',
            field=models.TextField(default='---', verbose_name='Abstract'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='meeting',
            name='contributions_message',
            field=models.TextField(blank=True, help_text='Message on contributions page, you can use Markdown here.', null=True, verbose_name='Contributions message'),
        ),
        migrations.AlterField(
            model_name='meeting',
            name='participants_message',
            field=models.TextField(blank=True, help_text='Message on participants page, you can use Markdown here.', null=True, verbose_name='Participants message'),
        ),
        migrations.AlterField(
            model_name='meeting',
            name='registration_done_message',
            field=models.TextField(blank=True, help_text='Message on the page displayed after registration, you can use Markdown here.', null=True, verbose_name='Registration done message'),
        ),
        migrations.AlterField(
            model_name='meeting',
            name='registration_message',
            field=models.TextField(blank=True, help_text='Message on registration page, you can use Markdown here.', null=True, verbose_name='Registration message'),
        ),
    ]
